@@ -66,6 +66,13 @@ static void base_add_xcode_docsets (DhBase      *base,
 
 static DhBase *base_instance;
 
+static void
+unref_node_link (GNode *node, gpointer data)
+{
+    dh_link_unref (node->data);
+    dh_link_unref (node->data);
+}
+
 static void
 base_finalize (GObject *object)
 {
@@ -74,6 +81,17 @@ base_finalize (GObject *object)
         priv = GET_PRIVATE (object);
 
         /* FIXME: Free things... */
+        g_hash_table_destroy (priv->books);
+        g_node_traverse (priv->book_tree,
+                         G_IN_ORDER,
+                         G_TRAVERSE_ALL,
+                         -1,
+                         (GNodeTraverseFunc)unref_node_link,
+                         NULL);
+        g_node_destroy (priv->book_tree);
+
+        g_list_foreach (priv->keywords, (GFunc)dh_link_unref, NULL);
+        g_list_free (priv->keywords);
 
         G_OBJECT_CLASS (dh_base_parent_class)->finalize (object);
 }
@@ -581,3 +599,16 @@ dh_base_get_window (DhBase *base)
 
         return window;
 }
+
+void
+dh_base_quit (DhBase *base)
+{
+        DhBasePriv *priv = GET_PRIVATE (base);
+
+        /* Make sure all of the windows get a chance to release their resources
+         * properly.  As they get destroyed,
+         * base_window_or_assistant_finalized_cb() will be called, and when the
+         * last one is removed, we will quit */
+        g_slist_foreach (priv->windows, (GFunc)gtk_widget_destroy, NULL);
+        g_slist_foreach (priv->assistants, (GFunc)gtk_widget_destroy, NULL);
+}
